      subroutine cfsXS(jdhost,kdhost,ldhost, jend,  kend,  lend,
     .                    jcor,  kcor,  lcor, X,Y,Z)

C     A routine to output an X array from many nodes to a single CFS file.
C     Plot3d format is used. 
C     The storage of X,Y, and Z in separate files corresponds to 
C     S. Weeratunga's parallel version of ARC3D.
C     Written and tested July 17, 1991 by Steve Ryan.

C     Local grid is:
C         1,jend           k,kend           l,lend           in local notation
C         jcor,jcor+jend-1 kcor,kcor+kend-1 lcor,lcor+lend-1 in global notation

      parameter(iprec=4,nids=5,iounit=10)

      real*4 X(jend,kend,lend),Y(jend,kend,lend),Z(jend,kend,lend)

      integer*4 id(0:nids-1)
      integer dimsize
      integer*4 ibuff(4)

      myn = mynode()

C     Size of header information for solution file
      dimsize  = 3


c     Node 0 creates and opens.  All sync.  Then nodes>0 open.
      if (myn.eq.0) then
        open(unit=iounit,file='/cfs/dirname/X.dat',status='unknown',
     .       form='unformatted')
        call gsync()
      else
        call gsync()
        open(unit=iounit,file='/cfs/dirname/X.dat',status='old',
     .       form='unformatted')
      end if

C     Calculate and set the file size in advance. This is critical for speed.
      isize =  dimsize*4 + (3*jdhost*kdhost*ldhost)*iprec
      idiag = lsize(iounit,isize,0)
      if (idiag .ne. isize) write(*,*) 'lsize failed.  Node: ',myn,
     .                  ' Desired and returned sizes: ',isize,idiag

C     Set up and fill the file headers (dimensions and flow quantities)
      if (myn.eq.0) then
        ibuff(1) = jdhost
        ibuff(2) = kdhost
        ibuff(3) = ldhost
	call lseek(iounit,0,0)
	call cwrite(iounit,ibuff,dimsize*4)
      end if

C     Prepare for writing "strips" of information to the disk
      jstep = 1
      kstep = jdhost
      lstep = jdhost*kdhost

      jsize = jend*iprec
      ksize = kstep*iprec
      lsiz  = lstep*iprec
      jklsize = jdhost*kdhost*ldhost*iprec

C     Write Grid file info:
C     ipos starts at the first point of the local grid, and is
C     incremented from there
      ipos  = 0
      ipos1 = dimsize*4
     .      + ((jcor-1)*jstep + (kcor-1)*kstep + (lcor-1)*lstep)*iprec

C     Use a rotating list of ids to keep track of up to "nids" 
C     asynchronous writes at a time.

      icount = 0
      if (myn.eq.0) write(*,*) 'Writing X to CFS'
      do 210 l = 1,lend
        ipos = ipos1 + (l-1)*lsiz
        do 200 k = 1,kend
          icount = icount+1
          idnum  = mod(icount,nids)
          if (icount.ge.nids) call iowait(id(idnum))
          call lseek(iounit,ipos,0)
          id(idnum) = iwrite(iounit, X(1,k,l), jsize)
          ipos = ipos + ksize
200     continue
210   continue
      if (myn.eq.0) write(*,*) 'Writing Y to CFS'
      ipos1 = ipos1 + jklsize
      do 310 l = 1,lend
        ipos = ipos1 + (l-1)*lsiz
        do 300 k = 1,kend
          icount = icount+1
          idnum  = mod(icount,nids)
          call iowait(id(idnum))
          call lseek(iounit,ipos,0)
          id(idnum) = iwrite(iounit, Y(1,k,l), jsize)
          ipos = ipos + ksize
300     continue
310   continue
      if (myn.eq.0) write(*,*) 'Writing Z to CFS'
      ipos1 = ipos1 + jklsize
      do 410 l = 1,lend
        ipos = ipos1 + (l-1)*lsiz
        do 400 k = 1,kend
          icount = icount+1
          idnum  = mod(icount,nids)
          call iowait(id(idnum))
          call lseek(iounit,ipos,0)
          id(idnum) = iwrite(iounit, Z(1,k,l), jsize)
          ipos = ipos + ksize
400     continue
410   continue

      do 530 mmm = 0,nids-1
530     call iowait(id(mmm))

      call gsync()
      close(iounit)

      return
      end
