      subroutine cfsioS2(jdhost,kdhost,ldhost, jend,  kend,  lend,
     .                    jcor,  kcor,  lcor, FSMACH,ALPHA,RE,TIME,Q)

C     A routine to output a Q array from many nodes to a single CFS file.
C     Plot3d format is used, except that this version of the routine
C     assumes that Q is indexed as (5,j,k,l) rather than (j,k,l,5).
C     Written and tested July 17, 1991 by Steve Ryan.

C     Local grid is:
C         1,jend           k,kend           l,lend           in local notation
C         jcor,jcor+jend-1 kcor,kcor+kend-1 lcor,lcor+lend-1 in global notation

      parameter(iprec=4,nids=5,iounit=10)

      real*4 Q(5,jend,kend,lend)

      integer stime,etime,dtime

      integer*4 id(0:nids-1)
      integer headsize,dimsize
      integer*4 ibuff(4)
      real*4     buff(4)

      myn = mynode()

C     Size of header information for solution file
      dimsize  = 3
      headsize = 4


c     Node 0 creates and opens.  All sync.  Then nodes>0 open.
      if (myn.eq.0) then
        open(unit=iounit,file='/cfs/dirname/Qnew.dat',status='unknown',
     .       form='unformatted')
        call gsync()
      else
        call gsync()
        open(unit=iounit,file='/cfs/dirname/Qnew.dat',status='old',
     .       form='unformatted')
      end if

C     Calculate and set the file size in advance. This is critical for speed.
      isize =  dimsize*4 + (headsize + 5*jdhost*kdhost*ldhost)*iprec
      idiag = lsize(iounit,isize,0)
      if (idiag .ne. isize) write(*,*) 'lsize failed.  Node: ',myn,
     .                  ' Desired and returned sizes: ',isize,idiag

C     Set up and fill the file headers (dimensions and flow quantities)
      if (myn.eq.0) then
        ibuff(1) = jdhost
        ibuff(2) = kdhost
        ibuff(3) = ldhost
	call lseek(iounit,0,0)
	call cwrite(iounit,ibuff,dimsize*4)
        buff(1) = FSMACH
        buff(2) = ALPHA
        buff(3) = RE
	buff(4) = TIME
	call cwrite(iounit,buff,headsize*iprec)
      end if

C     Prepare for writing "strips" of information to the disk
C     Each contains a factor of five, to include all five variables.
C       (in cfsio2, but not in cfsio1)
      jstep = 5
      kstep = jdhost*5
      lstep = jdhost*kdhost*5

      jsize = jend*iprec*5
      ksize = kstep*iprec
      lsiz  = lstep*iprec

C     Write Solution file info:
C     ipos starts at the first point of the local grid, and is
C     incremented from there
      ipos  = 0
      ipos1 = dimsize*4 + headsize*iprec  
     .      + ((jcor-1)*jstep + (kcor-1)*kstep + (lcor-1)*lstep)*iprec

C     Use a rotating list of ids to keep track of up to "nids" 
C     asynchronous writes at a time.

      if (myn.eq.0) write(*,*) 'starting to write solution to CFS'

C Enable the following lines start timing:
c      call gsync()
c      stime=mclock()

      icount = 0
      do 210 l = 1,lend
        ipos = ipos1 + (l-1)*lsiz
        do 200 k = 1,kend
          icount = icount+1
          idnum  = mod(icount,nids)
          if (icount.ge.nids) call iowait(id(idnum))
          call lseek(iounit,ipos,0)
          id(idnum) = iwrite(iounit, Q(1,1,k,l), jsize)
          ipos = ipos + ksize
200     continue
210   continue
      do 230 mmm = 0,nids-1
230     call iowait(id(mmm))


C Enable the following lines to output timing data to fort.012:
c      call gsync()
c
c      etime=mclock()
c
c      dtime = etime-stime
c      if (myn.eq.0) then
c	num = numnodes()
c	npts=jdhost*kdhost*ldhost
c	bps=float(npts*5*4)/float(dtime*1000)
c        write(12,2000) num,npts,dtime,jsize,bps
c        close(12)
c      end if
c2000  format('num=',I4,' total pts=',I8,
c     .       ' time=',I8,' msg size=',I6,' Mb/sec =',F10.3)


      call gsync()
      close(iounit)

      return
      end

