      program node
c     A demonstration program to calculate the temperature distribution
c     in a rectangle with imposed temperatures at the sides.
c     Assume that the points are equally spaced.
c
c     node will use only a fraction of the memory for the whole field,
c     working in the "bottom left" corner of its arrays, regardless of
c     the actual section of the problem it is using.
c
c               j->     T1
c              ____________________
c      ^       |                  |    
c      k    T2 |                  | T4
c              |__________________|
c                       T3
c
      include 'para.h'

C     lfake is the number of planes to use in creating a 3-D file
C     from 2-D data.
      parameter (lfake=4)

      real*4 T(NTHREE)
      real*4 Q(lfake*5*NTHREE)
      real*4 X(lfake*NTHREE),Y(lfake*NTHREE),Z(lfake*NTHREE)
      real*4 j1bdy(NONE),j2bdy(NONE),k1bdy(NONE),k2bdy(NONE)
      real*4 j1in(NONE),j2in(NONE),k1in(NONE),k2in(NONE)
      real*4 temp(NONE)
      real*4 resid,work

      integer*4 DONETYPE,INFOTYPE
      integer*4 INFOSIZE
      integer*4 JMPTYPE,JPMTYPE,KMPTYPE,KPMTYPE
      integer*4 HOSTPID,HOST
      integer*4 info(nodemax,8)

      integer stime,etime,dtime
      integer*4 id(0:nids-1)
      integer ids(maxsplit),jds(maxsplit),kds(maxsplit)
      integer size
      integer jpm_rid, jmp_rid, kpm_rid, kmp_rid 
      integer jpm_sid, jmp_sid, kpm_sid, kmp_sid 
      integer*4 itemp(NONE)


      data   HOSTPID /100/,
     .       INFOTYPE /1/,
     .	     DONETYPE /2/,
     .       JMPTYPE  /10/,
     .       JPMTYPE  /11/,
     .       KMPTYPE  /12/,
     .       KPMTYPE  /13/

c     initial weight
      wt = 1500.0
c      wt = 0.0

      INFOSIZE = nodemax*8*4

C     Receive and interpret the information about the size and location
C     of the grid which this node is responsible for.
      call crecv(INFOTYPE, info, INFOSIZE)
      HOST = myhost()
      n  = mynode()
      jend = info(n+1,1)
      kend = info(n+1,2)
      jmn  = info(n+1,3)-1
      jpn  = info(n+1,4)-1
      kmn  = info(n+1,5)-1
      kpn  = info(n+1,6)-1
      jcor = info(n+1,7)
      kcor = info(n+1,8)

      jendm = jend-1
      kendm = kend-1

      jbdysize = kend*iprec
      kbdysize = jend*iprec

      if (jend*kend .gt. NTHREE) then
	write(*,*) 'NTHREE is too small at node ',n
	write(*,*) 'NTHREE=',NTHREE,' jend*kend= ',jend*kend
	resid = -1.0
        if (n.eq.0) call csend(DONETYPE,resid,iprec,HOST,HOSTPID)
        stop
      end if

c     Initial conditions
      call init(T,jend,kend,jmn,jpn,kmn,kpn)


c     Iterations
      do 100 it = 1,maxit
        call calc(T,jend,kend,jendm,kendm,resid)

c       Don't start new messages unless previously sent ones have been received.
        if (it.gt.1) then
          if (kpn.ge.0) call msgwait(kpm_sid)
          if (jpn.ge.0) call msgwait(jpm_sid)
          if (kmn.ge.0) call msgwait(kmp_sid)
          if (jmn.ge.0) call msgwait(jmp_sid)
        end if

c       Get ready to receive any expected messages
        if (kpn.ge.0) kmp_rid = irecv(KMPTYPE, k2in, KBDYSIZE)
        if (jpn.ge.0) jmp_rid = irecv(JMPTYPE, j2in, JBDYSIZE)
        if (kmn.ge.0) kpm_rid = irecv(KPMTYPE, k1in, KBDYSIZE)
        if (jmn.ge.0) jpm_rid = irecv(JPMTYPE, j1in, JBDYSIZE)

c       Send new messages, but don't wait for them to be received.
c       Update the corners, even though they're not used
c       in the calculation.  They may the be used for other purposes (I/O).
        if (kpn.ge.0) then
	  call iomsg(1,T,1,jend,kendm,kendm,jend,kend,k2bdy)
	  kpm_sid = isend(KPMTYPE, k2bdy, KBDYSIZE, kpn ,mypid())
	end if
        if (jpn.ge.0) then
	  call iomsg(1,T,jendm,jendm,1,kend,jend,kend,j2bdy)
	  jpm_sid = isend(JPMTYPE, j2bdy, JBDYSIZE, jpn ,mypid())
	end if
        if (kmn.ge.0) then
	  call iomsg(1,T,1,jend,2,2,jend,kend,k1bdy)
	  kmp_sid = isend(KMPTYPE, k1bdy, KBDYSIZE, kmn ,mypid())
	end if
        if (jmn.ge.0) then
	  call iomsg(1,T,2,2,1,kend,jend,kend,j1bdy)
	  jmp_sid = isend(JMPTYPE, j1bdy, JBDYSIZE, jmn ,mypid())
	end if

	if (iprec.eq.4) then
	  call gssum(resid, 1, work)
	else if (iprec.eq.8) then
	  call gdsum(resid, 1, work)
	else
	  write(*,*) 'Problems!'
	  resid = -1.0
          if (n.eq.0) call csend(DONETYPE,resid,iprec,HOST,HOSTPID)
	  stop
	end if

        resid = sqrt(resid) / ((jdhost-1)*(kdhost-1))

	if (n.eq.0) then
         if (mod(it,iprint).eq.0) write(*,1001) it,resid
	 if (resid.le.target) goto 101
        end if

c       Use all incoming messages before going to the next iteration.
        if (jmn.ge.0) then
	  call msgwait(jpm_rid)
	  call iomsg(0,T,1,1,1,kend,jend,kend,j1in)
        end if
        if (jpn.ge.0) then
	  call msgwait(jmp_rid)
	  call iomsg(0,T,jend,jend,1,kend,jend,kend,j2in)
        end if
        if (kmn.ge.0) then
	  call msgwait(kpm_rid)
	  call iomsg(0,T,1,jend,1,1,jend,kend,k1in)
        end if
        if (kpn.ge.0) then
	  call msgwait(kmp_rid)
	  call iomsg(0,T,1,jend,kend,kend,jend,kend,k2in)
        end if

100   continue
101   continue
      write(*,*) 'after 101, ready to write solution'

C     Write the appropriate portion of the problem, then wait for
C     all nodes before sending done message.

C     Local grid is j=jcor,jcor+jendm,  k=kcor,kcor+kendm
C     This version just overwrites the points which overlap between
C     nodes.

C     Make a fake plot3d Q file from the 2-D, 1-variable data in the
C     program.  This version indexes Q as (5,jdim,kdim,ldim), so
C     it is not quite plot3d format, but it corresponds to the storage
C     method in Sisira Weeratunga's parallel version of ARC3D.
C     Normal (Q(jdim,kdim,ldim,5)) and double-precision versions of the
C     cfsio routine exist.

      call fakeQ(T,Q,jend,kend,lfake)
      call fakeX(X,Y,Z,jend,kend,lfake,jcor,kcor,1)

C     Write the grid to CFS
      call cfsXS(jdhost,kdhost,lfake,
     .             jend,  kend,lfake,
     .             jcor,  kcor,    1,  X,Y,Z)

C     Write the fake solution to CFS
      call cfsioS2(jdhost,kdhost,lfake,
     .               jend,  kend,lfake,
     .               jcor,  kcor,1,
     .      1.0,0.0,1000.0,1.0,Q)


C     Wait for all nodes to finish, the node zero signals the host.
      if (n.eq.0) then
        call gsync()
        call csend(DONETYPE,resid,iprec,HOST,HOSTPID)
      else 
        call gsync()
      end if
      

1001  format(I6,E10.2,2F12.3,I8)
      stop
      end
